"""
 (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
 All Rights Reserved

 URLSessionAdapter is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 URLSessionAdapter is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with URLSessionAdapter; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

# $Id: preamble.txt,v 1.1.1.1 2002/04/04 20:26:04 ender Exp $

from Globals import DTMLFile
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Interface import Base

class IURLSessionAdapter(Base):

    """

    This class provides for url identifiers
    for sessions. it integrates with the
    session tracking support in zope2.5+.
    It works in conjunction with an access rule
    which is also created upon instantiation, such
    that normal access rule suppression will also
    disable this.
    
    """

    def __call__ (self, request, response):
        """

        """

class URLSessionAdapter(SimpleItem, PropertyManager):

    meta_type = 'URLSesssionAdapter'

    __implements__ = (IURLSessionAdapter,)

    manage_options = (
        {'label':'Configuration',
         'action':'manage_propertiesForm',
         'help':('URLSessionAdapter', 'URLSessionAdapter.stx')
         },
        ) + SimpleItem.manage_options

    _properties = (
        {'id':'redirect_p', 'type':'int', 'mode':'w'},
        {'id':'active_p',   'type':'int', 'mode':'w'},        
        )

    ## class level defaults
    active_p = 1 # is the adapter active
    redirect_p = 1 # reload the page with the session in the url if new sess
    adapter_browser_namespace = 'form' # not really configurable 
    name_browser_id_manager = 'browser_id_manager' # id of the bid manager
    
    def __init__ (self, id):
        self.id = id

    def __call__(self, container, request):
        
        if not self.active_p:
            return
        
        browser_manager = getattr(self, self.name_browser_id_manager)

        stack = request['TraversalRequestNameStack']
        
        # check if traversing to container
        # with no existing session
        if not stack:
            id = None

        # don't intercept management screens
        elif stack[0][:6] == 'manage' or \
             stack[-1][:6] == 'manage':
            return

        # check if one exists in the url
        elif browser_manager._isAWellFormedBrowserId(stack[-1]):
            id = stack.pop()

        # no session yet
        else:
            id = None

        # setup a new session id
        if id is None:
            
            id = browser_manager._getNewBrowserId()
            
            if self.redirect_p:
                url = '/'.join([id]+request.path) # make it relative
                return request.RESPONSE.redirect(url)
            
            namespace = None
            
        else:
            namespace = self.adapter_browser_namespace
            
        # setup the virtual root
        request.setVirtualRoot(request.steps+[id])

        # setup session tracking info
        request.browser_id_    = id
        request.browser_id_ns_ = namespace
        
def manage_addURLSessionAdapter(self,
                                id = 'URLSessionAdapter',
                                redirect = 1,
                                create_access_rule_p = 1,
                                REQUEST=None
                                ):
    """
    add a urlsession adapter making the containing folder
    automatically use url sessions.
    """ 

    obj = URLSessionAdapter(id)
    obj.redirect = int(redirect)
    self._setObject(id, obj)

    if create_access_rule_p:
        self.manage_addProduct['SiteAccess'].manage_addAccessRule(
            method_id=id
            )

    if REQUEST:
        REQUEST.RESPONSE.redirect('manage_main')
    

manage_addURLSessionAdapterForm = DTMLFile('ui/addURLSessionAdapterForm',
                                           globals()
                                           )




